/***************************************************************************************************\
 *
 *	MINI TUTORIAL
 *
 *	RADIX SORT
 *
 *	Autor:
 *		Mauricio Lopez ( maur.lopez@gmail.com )
 *
 *	alias:
 *		skull, bret-hart, hit-man
 *
 *	Version 1.0 -- Agosto 2006
 *
\***************************************************************************************************/

/*
 *	1.  ORDENAMIENTO POR CONTEO
 *
 *	Conceptualmente el ordenamiento por conteo es simple de entender, ya que es natural para nosotros
 *	cuando ordenamos una lista de numeros mentalmente, por ejemplo, si tenemos la lista desordenada
 *	de numeros:
 *
 *	5 1 5 0 8 4 3 6 0 8 3 2
 *
 *	contamos las ocurrencias de cada numero en un array como se ve a continuacion
 *
 *	           0   1   2   3   4   5   6   7  8
 *           -------------------------------------
 *	contador | 2 | 1 | 1 | 2 | 1 | 2 | 1 | 0 | 2 |
 *	         -------------------------------------
 *
 *	luego solo hay que repetir las cuentas en orden para obtener la lista ordenada, y obtenemos la lista
 *	que mostramos a continuacion:
 *
 *	0 0 1 2 3 3 4 5 5 6 8 8
 *
 *	Este sencillo algoritmo requiere demasiados recursos, es necesario que el array contador sea lo
 *	suficientemente grande para indexar al mayor numero de la lista. Si a la lista desordenada
 *	previamente mostrada le aadimos el numero 99,999,999 necesitariamos que el array contador tubiera
 *	capacidad para 99,999,999 elementos solo para ordenar 13 numeros.
 *
 *	Por suerte es posible reducir considerablemente el uso de recursos si utilizamos un simple array
 *	de colas. El algoritmo de conteo aumenta su eficiencia y cambia de nombre para llamarse RADIX SORT.
 *
 *	2.  RADIX SORT
 *
 *	Supongamos que tenemos un arreglo de 10 colas, podemos modificar el algoritmo de conteo para
 *	insertar los numeros repetidos en una de las 10 colas. Este algoritmo, en escencia, sigue haciendo
 *	el conteo de elementos. Siguiendo con el ejemplo anterior, el arreglo de colas quedaria como
 *	se muestra a continuacion:
 *
 *	           0   1   2   3   4   5   6   7   8   9
 *           -----------------------------------------
 *	radix    | 0 | 1 | 2 | 3 | 4 | 5 | 6 | * | 8 | * |
 *	         -----------------------------------------
 *	         | 0 | * | * | 3 | * | 5 | * | * | 8 | * |
 *	         -----------------------------------------
 *
 *	Al extraer secuencialmente los numeros de las colas, obtenemos la lista ordenada:
 *
 *	0 0 1 2 3 3 4 5 5 6 8 8
 *
 *	Este algoritmo funciona bien para listas con numeros de un solo digito. Para ordenar listas de mas
 *	digitos no es necesario aumentar el numero de colas, en cambio, la filosofia del algoritmo es ordenar
 *	la lista primero basandonos en el digito menos significativo (unidades), hasta los ms significativos
 *	(decenas, centenas, millares, etc.). Es decir que, por ejemplo, para ordenar listas de dos digitos, 
 *	el algoritmo necesitaria hacer solo dos iteraciones. Digamos que tenemos la lista que se ve a
 *	continuacion:
 *
 *	10 25 36 02 36 09 99 48 20 65
 *
 *	En la primera iteracion ordenamos la lista basandonos en los digitos menos significativos (unidades):
 *
 *	           0    1   2    3   4   5    6    7   8    9
 *           -----------------------------------------------
 *	radix    | 10 | * | 02 | * | * | 25 | 36 | * | 58 | 09 |
 *	         -----------------------------------------------
 *	         | 20 | * | *  | * | * | 65 | *  | * | 48 | 99 |
 *	         -----------------------------------------------
 *
 *	la lista queda ordenada por el digito menos significativo, tal como se muestra a continuacion:
 *
 *	10 20 02 25 65 36 58 48 09 99
 *
 *	la segunda iteracion muestra como el algoritmo ordena completamente la lista, esto es debido a que
 *	el ordenamiento se da en forma "estable", es decir, los numeros ingresan ordenadamente a las colas
 *	debido al orden dado en la iteracion anterior
 *
 *	           0    1    2    3    4   5   6    7   8    9
 *           -------------------------------------------------
 *	radix    | 02 | 10 | 20 | 36 | 48 | 58 | 65 | * | * | 99 |
 *	         -------------------------------------------------
 *	         | 09 | *  | 25 | *  | *  | *  | *  | * | * | *  |
 *	         -------------------------------------------------
 *
 *	Al extraer secuencialmente los numeros de las colas, obtenemos la lista ordenada:
 *
 *	02 09 10 20 25 36 48 58 65 99
 *
 *	Este algoritmo puede ordenar numeros de N digitos en N iteraciones, usando 10 colas (radix = 10),
 *	sin embargo, el algoritmo tambien puede trabajar con solo dos colas, ordenando bit por bit,
 *	o en base hexadecimal con 16 colas. En la practica el algoritmo radix se puede optimizar si usamos
 *	bases potencia de dos: radix = 16 o bien radix = 256. En especial resulta conveniente usar radix = 256
 *	(256 colas), debido a que es usual que los numeros a ordenar sean enteros de 4 bytes o bien
 *	flotantes de 4 bytes y podriamos ordenar cualquier lista en solo 4 iteraciones (ordenando byte
 *	por byte).
 *
 *	El siguiente algoritmo ejemplifica el ordenamiento de numeros enteros de 4 bytes:
 *	
	//vec : vector a ordenar
	//n   : numero de elementos del vector
	void	radix_sort( unsigned int vec[], const int n )
	{
		int					i, bytes, peso, j, k;
		std::queue<int>		radix[256]; //requerimos 256 colas dinamicas

		bytes = 4;  //numeros de 32 bits
		peso = 0;
		for( k = 0 ; k < bytes ; ++k )
		{
			//inserta los numeros en las colas, ordenando segun el byte que corresponda
			for( i = 0 ; i < n ; ++i )
			{
				//extrae un byte del numero que esta en vec[i]
				j = ( vec[ i ] >> (peso*8) ) & 0xFF;

				//inserta el numero en la cola correspondiente
				radix[ j ].push( vec[ i ] );
			}

			i = 0;

			//extrae secuencialmente los numeros de las colas
			for( j = 0 ; j < 256 ; ++j )
			{
				while( !radix[ j ].empty() )
				{
					vec[ i++ ] = radix[ j ].front();
					radix[ j ].pop();
				}
			}

			//se prepara para ordenar por el siguiente byte
			peso++;
		}
	}
 *
 *	El ordenamiento de numeros flotantes de 4 bytes es posible slo si antes se transforman a 
 *	enteros usando la siguiente funcion:
 *
	unsigned int	flotanteAEntero( float f )
	{
		unsigned int	d = *( (unsigned int*)( (void*)(&f) ) );
		return ( d ^ ((-(d >> 31)) | 0x80000000) );
	}
 *
 *	RADIX SORT OPTIMIZADO
 *
 *	Si bien el algoritmo clasico radix es muy eficiente, eliminar el uso de colas optimizaria aun
 *	el algoritmo, ya que se elimiaria el tiempo que toman las operaciones push y pop. A continuacin
 *	describir una forma de hacerlo, segn el algoritmo de Huy T. Vo, de la universidad de UTAH.
 *
 *	En lugar de usar las 256 colas, el algoritmo de Huy Vo requiere un arreglo temporal del mismo
 *	tamao que el arreglo a ordenar. Como es de esperar, las inserciones al arreglo son mas eficientes
 *	que a las colas, ya que stas deben ser dinmicas, lo cual implica reservar y liberar memoria
 *	dinamica repetidas veces.
 *
 *	Me resultar mas sencillo explicar el algoritmo con un ejemplo concreto.
 *
 *	Supongamos que tenemos un arreglo de numeros como se muestra a continuacion:
 *
 *	5 0 5 8 4 3 7 8 6 0 8 4 3 7
 *
 *	Intencionalmente he creado una lista con slo numeros de un digito y varios digitos repetidos. Esto
 *	simplificar la explicacin, sin embargo, el algoritmo no pierde generalidad.
 *
 *	El primer paso del algoritmo es realizar un conteo, tal cual lo hace el ordenamiento por conteo:
 *
 *	           0   1   2   3   4   5   6   7   8   9
 *           -----------------------------------------
 *	contador | 2 | 0 | 0 | 2 | 2 | 2 | 1 | 2 | 3 | 0 |
 *	         -----------------------------------------
 *
 *	Sin embargo la intencion no es ordenar por conteo, sino crear un indice que nos permita reordenar
 *	la lista tal como si hubiera sido insertada en 10 colas del algoritmo radix clasico. La creacion del
 *	indice es muy sencilla. Como se muestra a continuacion el indice es un arreglo de 10 elementos que
 *	inicialmente esta vacio y tiene el digito cero en la primera posicion:
 *
 *	           0   1   2   3   4   5   6   7   8   9
 *           -----------------------------------------
 *	indice   | 0 | * | * | * | * | * | * | * | * | * |
 *	         -----------------------------------------
 *
 *	El indice se crea utilizando el array contador. La intencin es calcular las posiciones que ocupar
 *	cada numero en el arreglo ordenado. Podemos hacerlo de forma incremental de la siguiente manera:
 *
 *	La posicion i del contador se suma con la posicion i del indice, el resultado se guarda en la
 *	posicion i+1 del indice, formalmente:
 *
 *	indice[ 0 ] = 0;
 *	for( i = 1 ; i < 10 ; ++i )
 *		indice[ i ] = contador[ i - 1 ] + indice[ i - 1 ];
 *
 *	Esto lo podemos apreciar en el siguiente esquema:
 *
 *	           0   1   2   3   4   5   6   7   8    9
 *           -------------------------------------------
 *	indice   | 0 | 2 | 2 | 2 | 4 | 6 | 8 | 9 | 11 | 14 |
 *	         -------------------------------------------
 *             +   +   +   +   +   +   +   +   +
 *           -----------------------------------------
 *	contador | 2 | 0 | 0 | 2 | 2 | 2 | 1 | 2 | 3 | 0 |
 *	         -----------------------------------------
 *
 *	Usando este indice podemos ordenar la lista en un arreglo temporal como se muestra a continuacion:
 *
	for( i = 0 ; i < N ; ++i )
	{
		temporal[ indice[ lista[i]  ] ] = lista[i];
		indice[ lista[i]  ]++;
	}
 *
 *	Ilustrando: si lista[0] es 5, entonces le corresponde el sitio 6 en el arreglo temporal, luego
 *	incrementamos el indice para que al siguiente 5 le corresponda el sitio 7. En el siguiente esquema
 *	apreciamos como el indice posiciona cada numero en el arreglo temporal, por claridad las posiciones
 *	donde van numeros repetidos estan tapadas con un asterisco. Los repetidos tienen su lugar reservado
 *	en el arreglo, ya que despues de un acceso al indice ste es incrementado en uno, los repetidos
 *	se insertan adecuadamente uno despues del otro.
 *
 *           ---------------------------------------------------------
 *	lista    | 5 | 0 | 5 | 8 | 4 | 3 | 7 | 8 | 6 | 0 | 8 | 4 | 3 | 7 |
 *	         ---------------------------------------------------------
 *
 *	           0   1   2   3   4   5   6   7    8    9
 *           --------------------------------------------
 *	indice   | 0 | 2 | 2 | 2 | 4 | 6 | 8 | 9  | 11 | 14 |
 *	         --------------------------------------------
 *
 *           ---------------------------------------------------------
 *	temporal | 0 | * | 3 | * | 4 | * | 5 | * | 6 | 7 | * | 8 | * | * |
 *	         ---------------------------------------------------------
 *             0   1   2   3   4   5   6   7   8   9  10  11  12  13
 *
 *	Finalmente, que sucede si tenemos numeros de dos o mas digitos? al igual que radix clasico
 *	podemos iterar el ordenamiento digito por digito para obtener un ordenamiento total despues de N
 *	iteraciones. Al igual que en el radix clasico, resulta practico ordenar listas de numeros enteros
 *	byte por byte (radix = 256). A continuacion presentamos el algoritmo optimizado de Huy Vo:
 *
	void	Fradix(int byte, int len, unsigned int source[], unsigned int dest[], int count[]) 
	{
		register unsigned int	i;
		  
		static int index[256];

		index[ 0 ] = 0;
		for ( i = 1 ; i < 256 ; ++i )
			index[ i ] = index[ i - 1 ] + count[ i - 1 ];

		for ( i = 0 ; i < (unsigned int)len ; ++i ) 
		{
			int		j = ( source[ i ] >> (byte*8) ) & 0xFF;

			dest[ index[ j ] ] = source[ i ];
			index[ j ]++
		}
	}
 
	void	FradixSort( unsigned int *array, unsigned int *temp, int lo, int up )
	{
		register int			len = up - lo;
		register unsigned int	i, u;
		  
		unsigned int *uints = array + lo;
		  
		int			count[4][256] = {0};
		  
		// Generate count arrays
		for ( i = 0 ; i < (unsigned int)len ; ++i ) 
		{
			u = uints[i];
			count[0][ u & 0xff ]++;
			count[1][ (u >> 8) & 0xff ]++;
			count[2][ (u >> 16) & 0xff ]++;
			count[3][ (u >> 24) & 0xff ]++;
		}

		// Start sorting
		Fradix( 0, len, uints, temp, count[0] );
		Fradix( 1, len, temp, uints, count[1] );
		Fradix( 2, len, uints, temp, count[2] );
		Fradix( 3, len, temp, uints, count[3] );
	}
 *
 *	Al igual que el radix clasico, el radix optimizado solo requiere de 4 iteraciones (en este caso, 
 *	llamadas a funcion) para ordenar arreglos de cualquier tamao, tampoco usa menos memoria que el
 *	algoritmo radix clasico, sin embargo no requiere de 256 colas dinamicas, sino de un solo arreglo 
 *	temporal que tiene el mismo tamao que el arreglo a ordenar. Esto tiende a aumentar la velocidad
 *	del algoritmo optimizado frente al clasico.
 *
 *	Bueno, con esto finalizo este mini tutorial. Anexo a este tuto el codigo original del algoritmo
 *	radix de Huy T. Vo, el cual tiene licencia BSD Open Source y cuyos creditos son de la universidad
 *	de UTAH.
 *	
 */
#include "stdafx.h"
#include <conio.h>
#include "sort.h"
#include <queue>

/***************************************************************************************************\
 *
 *	Funcion creada para este mini tutorial:
 *		void	radix_sort( unsigned int vec[], int n );
 *
 *	Autor:
 *		Mauricio Lopez ( maur.lopez@gmail.com )
 *
 *	alias:
 *		skull, bret-hart, hit-man
 *
 *	Version 1.0 -- Agosto 2006
 *
\***************************************************************************************************/
void	radix_sort( unsigned int vec[], int n )
{
	using namespace		std;

	register int			i, bytes, peso, J, K;
	queue<unsigned int>		radix[256];

	bytes = 4;  //numeros de 32 bits
	peso = 0;
	for ( K = 0 ; K < bytes ; ++K )
	{
		for ( i = 0 ; i < n ; ++i )
		{
			J = ( vec[ i ] >> (peso*8) ) & 0xFF;
			radix[ J ].push( vec[ i ] );
		}

		i = 0;

		for ( J = 0 ; J < 256 ; ++J )
		{
			while ( !radix[ J ].empty() )
			{
				vec[ i++ ] = radix[ J ].front();
				radix[ J ].pop();
			}
		}
		peso++;
	}
}

unsigned int	flotanteAEntero( float f )
{
	unsigned int	d = *( (unsigned int*)( (void*)(&f) ) );
	return ( d ^ ((-(d >> 31)) | 0x80000000) );
}

#define		MAX		20

int _tmain(int argc, _TCHAR* argv[])
{
	/*
	 *	prueba el algoritmo radix original
	 */

	printf( "PRUEBA DEL ALGORITMO RADIX SORT\n\n" );
	printf( "presione una tecla........\n" );
	getch();

	int				array[MAX];// = { 10, 25, 36, 2, 36, 9, 99, 48, 20, 65 };

	for( int f = 0 ; f < MAX ; ++f )
	{
		array[ f ] =  MAX - f;
		printf( "(%i)\n", array[f] );
	}

	getch();

	radix_sort( (unsigned int*)array, MAX );

	printf( "\nLISTA ORDENADA\n\n" );

	for( int f = 0 ; f < MAX ; ++f )
	{
		printf( "(%i)\n", array[f] );
	}

	printf( "presione una tecla........\n" );
	getch();

	/*
	 *	prueba el algoritmo radix optimizado
	 */

	printf( "\nPRUEBA DEL ALGORITMO RADIX SORT OPTIMIZADO\n\n" );
	printf( "presione una tecla........\n" );
	getch();

	SortedFace		array2[MAX];
	SortedFace		arrayTMP[MAX];

	float		inc = 1.45f;

	for( int f = 0 ; f < MAX ; ++f )
	{
		float			fidx = float(MAX) - inc*f;

		SortedFace		face( MAX - f, *( (unsigned int*)( (void*)(&fidx) ) ) );

		array2[ f ] = face;

		printf( "(%u)\n", array2[ f ].mFace );
	}

	printf( "presione una tecla........\n" );
	getch();

	FradixSort( array2, arrayTMP, 0, MAX );

	printf( "\nLISTA ORDENADA\n\n" );

	for( int f = 0 ; f < MAX ; ++f )
	{
		printf( "(%u)\n", array2[ f ].mFace );
	}

	printf( "presione una tecla........\n" );
	getch();

	return 0;
}

